@@ -1,12 +1,10 @@
 package POE::Component::Server::SimpleHTTP;
+$POE::Component::Server::SimpleHTTP::VERSION = '2.28';
+#ABSTRACT: Perl extension to serve HTTP requests in POE.
 
 use strict;
 use warnings;
 
-use vars qw($VERSION);
-
-$VERSION = '2.20';
-
 use POE;
 use POE::Wheel::SocketFactory;
 use POE::Wheel::ReadWrite;
@@ -14,7 +12,7 @@ use POE::Filter::HTTPD;
 use POE::Filter::Stream;
 
 use Carp qw( croak );
-use Socket;
+use Socket qw( AF_INET AF_INET6 INADDR_ANY IN6ADDR_ANY inet_pton );
 
 use HTTP::Date qw( time2str );
 
@@ -43,6 +41,10 @@ has 'alias' => (
   is => 'ro',
 );
 
+has 'domain' => (
+  is => 'ro',
+);
+
 has 'address' => (
   is => 'ro',
 );
@@ -252,8 +254,10 @@ event 'SHUTDOWN' => sub {
    # Are we gracefully shutting down or not?
    if ( $graceful ) {
 
-      # Check for number of requests
-      if ( keys( %{ $self->_requests } ) == 0 ) {
+      # Check for existing requests and keep-alive connections
+      if ( keys( %{ $self->_requests } ) == 0
+         and keys( %{ $self->_connections } ) == 0 )
+      {
 
          # Alright, shutdown anyway
 
@@ -315,18 +319,39 @@ event 'start_listener' => sub {
 
       $self->inc_retry unless $noinc;
 
+      my $domain = $self->domain;
+      my $bindaddress = $self->address;
+      if ( not defined $bindaddress and not defined $domain ) {
+         $domain = AF_INET6;
+         $bindaddress = IN6ADDR_ANY;
+      } elsif ( not defined $bindaddress ) {
+         if ( $domain == AF_INET6 ) {
+            $bindaddress = IN6ADDR_ANY;
+         } elsif ( $domain == AF_INET ) {
+            $bindaddress = INADDR_ANY;
+         }
+      } else {
+         if ( defined inet_pton(AF_INET6, $bindaddress) ) {
+            $domain = AF_INET6;
+         } elsif ( defined inet_pton(AF_INET, $bindaddress) ) {
+            $domain = AF_INET;
+         }
+      }
+
       # Create our own SocketFactory Wheel :)
       my $factory = POE::Wheel::SocketFactory->new(
+         ( $domain ? ( SocketDomain => $domain ) : () ),
+         ( $bindaddress ? ( BindAddress => $bindaddress ) : () ),
          BindPort     => $self->port,
-         ( $self->address ? ( BindAddress => $self->address ) : () ),
          Reuse        => 'yes',
          SuccessEvent => 'got_connection',
          FailureEvent => 'listener_error',
       );
 
-      my ( $port, $address ) =
-        sockaddr_in( $factory->getsockname );
-      $self->_set_port( $port ) if $self->port == 0;
+      my ( $family, $address, $port, $straddress ) =
+      POE::Component::Server::SimpleHTTP::Connection->get_sockaddr_info(
+         $factory->getsockname );
+      $self->_set_port( $port ) if ( $self->port == 0 and $port );
 
       $self->_set_factory( $factory );
 
@@ -475,8 +500,11 @@ sub MassageHandlers {
 # 'Got_Connection'
 # The actual manager of connections
 event 'got_connection' => sub {
-   my ($kernel,$self,$socket,$peeraddr,$peerport) = @_[KERNEL,OBJECT,ARG0..ARG2];
+   my ( $kernel, $self, $socket ) = @_[KERNEL, OBJECT, ARG0];
 
+   my ( $family, $address, $port, $straddress ) =
+   POE::Component::Server::SimpleHTTP::Connection->get_sockaddr_info(
+      getpeername($socket) );
 
    # Should we SSLify it?
    if ( $self->sslkeycert ) {
@@ -484,9 +512,7 @@ event 'got_connection' => sub {
       # SSLify it!
       eval { $socket = Server_SSLify($socket) };
       if ($@) {
-         warn "Unable to turn on SSL for connection from "
-           . Socket::inet_ntoa( $peeraddr )
-           . " -> $@";
+         warn "Unable to turn on SSL for connection from $straddress -> $@";
          close $socket;
          return 1;
       }
@@ -796,11 +822,11 @@ sub must_keepalive {
    return 0 if $resp->is_error;
 
    # Connection is a comma-seperated header
-   my $conn = lc $req->header('Connection');
+   my $conn = lc ($req->header('Connection') || '');
    return 0 if ",$conn," =~ /,\s*close\s*,/;
-   $conn = lc $req->header('Proxy-Connection');
+   $conn = lc ($req->header('Proxy-Connection') || '');
    return 0 if ",$conn," =~ /,\s*close\s*,/;
-   $conn = lc $resp->header('Connection');
+   $conn = lc ($resp->header('Connection') || '');
    return 0 if ",$conn," =~ /,\s*close\s*,/;
 
    # HTTP/1.1 = keep
@@ -1174,10 +1200,18 @@ __PACKAGE__->meta->make_immutable( );
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
 POE::Component::Server::SimpleHTTP - Perl extension to serve HTTP requests in POE.
 
+=head1 VERSION
+
+version 2.28
+
 =head1 SYNOPSIS
 
 	use POE;
@@ -1311,10 +1345,6 @@ POE::Component::Server::SimpleHTTP - Perl extension to serve HTTP requests in PO
 		return;
 	}
 
-=head1 ABSTRACT
-
-	An easy to use HTTP daemon for POE-enabled programs
-
 =head1 DESCRIPTION
 
 This module makes serving up HTTP requests a breeze in POE.
@@ -1358,12 +1388,22 @@ This will default to "SimpleHTTP"
 
 =item C<ADDRESS>
 
-This value will be passed to POE::Wheel::SocketFactory to bind to, will use INADDR_ANY if it is nothing is provided.
+This value will be passed to POE::Wheel::SocketFactory to bind to, will use
+INADDR_ANY if it is nothing is provided (or IN6ADDR_ANY if DOMAIN is AF_INET6).
+For UNIX domain sockets, it should be a path describing the socket's filename.
+
+If neither DOMAIN nor ADDRESS are specified, it will use IN6ADDR_ANY and
+AF_INET6.
 
 =item C<PORT>
 
 This value will be passed to POE::Wheel::SocketFactory to bind to.
 
+=item C<DOMAIN>
+
+This value will be passed to POE::Wheel::SocketFactory to define the socket
+domain used (AF_INET, AF_INET6, AF_UNIX).
+
 =item C<HOSTNAME>
 
 This value is for the HTTP::Request's URI to point to.
@@ -1692,6 +1732,19 @@ For those who are pondering about basic-authentication, here's a tiny snippet to
 
 Nothing.
 
+=for Pod::Coverage        MassageHandlers
+       START
+       STOP
+       fix_headers
+       getsockname
+       must_keepalive
+       session_id
+       shutdown
+
+=head1 ABSTRACT
+
+	An easy to use HTTP daemon for POE-enabled programs
+
 =head1 SEE ALSO
 
 	L<POE>
@@ -1712,13 +1765,13 @@ Nothing.
 
 =head1 AUTHOR
 
-Apocalypse E<lt>apocal@cpan.orgE<gt>
+Apocalypse <APOCAL@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2006 by Apocalypse
+This software is copyright (c) 2018 by Apocalypse, Chris Williams, Eriam Schaffter, Marlon Bailey and Philip Gwyn.
 
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
 =cut
